@@ -40,11 +40,19 @@ def format_request_body(fields):
         return fields
 
 
-def upload(destination, file, progress_bar=False):
+def upload(destination, file, progress_bar=False, filename='features_backup.rcdb', ):
+    """
+
+    :param dict destination: an S3 presigned upload object
+    :param file: File-like object that has read
+    :param progress_bar: display progress bar or not
+    :param str filename: name of file when doing the upload
+    :return:
+    """
     url = destination["url"]
     fields = format_request_body(destination['fields'])
 
-    fields["file"] = ("features_backup.rcdb", file)
+    fields["file"] = (filename, file)
     encoder = MultipartEncoder(fields=fields)
     if not progress_bar:
         return requests.post(
@@ -58,23 +66,25 @@ def upload(destination, file, progress_bar=False):
         )
 
 
-def download(source, destination=None):
+def download(source, destination=None, progress_bar=False):
     is_file = False
     if destination is None:
         fp = six.BytesIO()
-    elif isinstance(destination, six.BytesIO) or isinstance(destination, six.StringIO):
+    elif hasattr(destination, 'write') and hasattr(destination, 'read'):
+        # it is a file-like object
         fp = destination
     else:
         fp = open(destination, 'wb')
         is_file = True
 
     r = requests.get(source, stream=True)
-    file_size = int(r.headers['Content-Length'])
     chunk, chunk_size = 1, 1024
-    with ProgressBar(total=file_size, unit='bytes', unit_scale=True, leave=False) as bar:
-        for chunk in ProgressBar(
-            r.iter_content(chunk_size)
-        ):
+
+    if progress_bar:
+        for chunk in ProgressBar(r.iter_content(chunk_size)):
+            fp.write(chunk)
+    else:
+        for chunk in r.iter_content(chunk_size):
             fp.write(chunk)
 
     if is_file:
