@@ -1,3 +1,5 @@
+from datetime import datetime
+
 import pandas as pd
 import numpy as np
 
@@ -23,6 +25,7 @@ def make_planogram_df(scan_data, planogram_number=None):
     bay_map = {
         sb["id"]: sb for sb in scan_data['shelf_bays']
     }
+    store = scan_data['store']
 
     flattened = []
     for item in scan_data['items']:
@@ -30,6 +33,7 @@ def make_planogram_df(scan_data, planogram_number=None):
         flattened.append({
             'planogram_number': planogram_number,
             'planogram_title': planogram_number,
+            'store_id': store.get('custom_id'),
             'category_number': scan_data.get('category_number'),
             'category_title': scan_data.get('category_title'),
             'bay_report_id': item['shelf_bay_id'],
@@ -64,4 +68,9 @@ def make_planogram_df(scan_data, planogram_number=None):
     df['facings_wide'] = 1
     df['facings_deep'] = 1
     df['facing_type'] = 'item'
+    df['section_direction'] = 'left_to_right'
+    df['shelf_direction'] = 'top_to_down'
+    df['position_direction'] = 'left_to_right'
+    df['from_date'] = datetime.now().strftime("%Y-%m-%d")
+    df['is_zig_zag'] = 0
     return df
