@@ -1,7 +1,9 @@
+# coding=utf-8
 import datetime
 import json
 import os
 import pathlib
+import tempfile
 from functools import partial
 from multiprocessing.pool import Pool
 
@@ -11,9 +13,13 @@ from prettytable import PrettyTable
 
 from rebotics_sdk.advanced import remote_loaders
 from rebotics_sdk.cli.common import shell, configure, roles
+from .renderers import format_full_table
 from .utils import ReboticsCLIContext, app_dir, pass_rebotics_context, read_saved_role, process_role
-from ..advanced.packers import ClassificationDatabasePacker, ClassificationDatabaseException
+from ..advanced.packers import ClassificationDatabasePacker, ClassificationDatabaseException, \
+    DuplicateFeatureVectorsException, VirtualClassificationDatabasePacker
+from ..constants import RCDB
 from ..providers import AdminProvider, RetailerProvider, sleep
+from ..advanced.flows import FileUploadError, RCDBImportFlow, download_vrcdb_file, ReboticsClassificationDatabase
 from ..providers.facenet import FacenetProvider
 from ..utils import Timer
 
@@ -223,36 +229,15 @@ def set_retailer_url(ctx, retailer, url):
         click.echo('Set new host for retailer %s' % retailer)
 
 
-def upload_packed(ctx, import_request, packed):
-    destination = import_request['destination']
-
-    if ctx.verbose:
-        click.echo("Sending the file to the S3 {}".format(destination))
-
-    response = remote_loaders.upload(destination, packed, progress_bar=True)
-
-    if response.status_code < 300:
-        if ctx.verbose:
-            click.echo("S3 {} response: {}".format(response.status_code, response.content))
-
-        ctx.provider.notify_classification_database_import_done(
-            id=import_request['id']
-        )
-        click.echo('File was uploaded to destination. Classification data backup ID={}'.format(import_request['id']))
-    else:
-        raise click.ClickException("Failed to upload classification file. Destination status: {}".format(
-            response.status_code)
-        )
-
-
 @api.command()
 @click.option('-r', '--retailer', help="Retailer codename", prompt=True)
 @click.option('-m', '--model', help="Model codename", prompt=True)
 @click.option('-i', '--images', type=click.Path(exists=True, file_okay=False, dir_okay=True))
 @click.option('-l', '--labels', type=click.Path(exists=True, file_okay=True, dir_okay=False))
 @click.option('-f', '--features', type=click.Path(exists=True, file_okay=True, dir_okay=False))
+@click.option('-c', '--check-duplicates', is_flag=True, help="MD5 checker simple")
 @pass_rebotics_context
-def pack_and_import_classification_db(ctx, retailer, model, images, labels, features):
+def pack_and_import_classification_db(ctx, retailer, model, images, labels, features, check_duplicates):
     """Pack classification database to single .rcdb file and import it to the Rebotics Admin
 
 \b
@@ -269,31 +254,39 @@ admin pack-classification-db --features features.txt --labels labels.txt --image
 
 admin import_classification_db --retailer delta --model test_code classification.rcdb
     """
-    packer = ClassificationDatabasePacker(destination=None, progress_bar=True)
-    if ctx.verbose:
-        click.echo("Packing from provided values: \n"
-                   "labels: {labels} \n"
-                   "features: {features} \n"
-                   "images: {images}".format(labels=labels, features=features, images=images))
-    try:
-        packed = packer.pack(labels, features, images)
-        packed.seek(0)
-    except ClassificationDatabaseException as exc:
-        raise click.ClickException(str(exc))
+    with RCDBImportFlow(ctx.provider, retailer, model, ClassificationDatabasePacker.extension) as flow:
+        if ctx.verbose:
+            click.echo("Creating import request: \n"
+                       "retailer: {}\n"
+                       "model: {}\n"
+                       "extension: {}".format(retailer, model, ClassificationDatabasePacker))
 
-    if ctx.verbose:
-        click.echo("Creating import request: \n"
-                   "retailer: {}\n"
-                   "model: {}\n"
-                   "extension: {}".format(retailer, model, packer.extension))
+        packer = ClassificationDatabasePacker(destination=None, progress_bar=True, check_duplicates=check_duplicates)
+        if ctx.verbose:
+            click.echo("Packing from provided values: \n"
+                       "labels: {labels} \n"
+                       "features: {features} \n"
+                       "images: {images}".format(labels=labels, features=features, images=images))
 
-    import_request = ctx.provider.create_classification_database_import(
-        retailer=retailer, model=model, extension=packer.extension
-    )
-    if ctx.verbose:
-        click.echo("Import request: {}".format(import_request))
+        try:
+            packed = packer.pack(labels, features, images)
+            packed.seek(0)
+        except DuplicateFeatureVectorsException as exc:
+            handle_duplicates(ctx, exc.duplicates)
+            raise click.ClickException(str(exc))
+        except ClassificationDatabaseException as exc:
+            raise click.ClickException(str(exc))
 
-    upload_packed(ctx, import_request, packed)
+        if ctx.verbose:
+            click.echo("Import request: {}".format(flow.import_request))
+
+        try:
+            flow.upload_file(packed, progress_bar=True)
+        except FileUploadError as exc:
+            click.echo(str(exc))
+            raise
+
+        click.echo("Import completed. Reference database id: {}".format(flow.import_request['id']))
 
 
 @api.command()
@@ -314,22 +307,50 @@ def import_classification_db(ctx, retailer, model, filepath):
                    "model: {}\n"
                    "extension: {}".format(retailer, model, extension))
 
-    import_request = ctx.provider.create_classification_database_import(
-        retailer=retailer, model=model, extension=extension
-    )
-    if ctx.verbose:
-        click.echo("Import request: {}".format(import_request))
-    with open(filepath, 'rb') as packed:
-        upload_packed(ctx, import_request, packed)
+    with RCDBImportFlow(ctx.provider, retailer, model, extension) as flow:
+        if ctx.verbose:
+            click.echo("Import request: {}".format(flow.import_request))
+        with open(filepath, 'rb') as packed:
+            flow.upload_file(packed, progress_bar=True)
+
+    click.echo("Import completed. Reference database id: {}".format(flow.import_request['id']))
+
+
+def handle_duplicates(ctx, duplicates):
+    unpacked = [
+        [
+            {
+                'id': entry.index,
+                'label': entry.label,
+                'filename': entry.filename
+            }
+            for entry in group
+        ] for group in duplicates
+    ]
+    if ctx.format == 'json':
+        click.echo(json.dumps(unpacked, indent=2))
+    elif ctx.format == 'table':
+        for group in unpacked:
+            format_full_table(group)
+    elif ctx.format == 'id':
+        click.echo("Duplicate id. New line separated for group; space separated inside of the group:")
+        for group in duplicates:
+            click.echo(" ".join([str(entry.index) for entry in group]))
+    else:
+        for group in duplicates:
+            for entry in group:
+                click.echo("{entry.index} {entry.label} {entry.filename}".format(entry=entry))
+            click.echo('=' * 100)
 
 
 @api.command()
 @click.option('-i', '--images', type=click.Path(exists=True, file_okay=False, dir_okay=True))
 @click.option('-l', '--labels', type=click.Path(exists=True, file_okay=True, dir_okay=False))
 @click.option('-f', '--features', type=click.Path(exists=True, file_okay=True, dir_okay=False))
+@click.option('-c', '--check-duplicates', is_flag=True, help="MD5 checker simple")
 @click.argument('target', type=click.Path(exists=False, file_okay=True, dir_okay=True), default='.')
 @pass_rebotics_context
-def pack_classification_db(ctx, images, labels, features, target):
+def pack_classification_db(ctx, images, labels, features, target, check_duplicates):
     """Pack classification database to single .rcdb file
 
 \b
@@ -350,7 +371,11 @@ db/
             datetime.datetime.now().strftime('%Y-%m-%dZ%H%M'),
             ClassificationDatabasePacker.extension
         )
-    packer = ClassificationDatabasePacker(destination=target, progress_bar=True)
+    packer = ClassificationDatabasePacker(
+        destination=target,
+        progress_bar=True,
+        check_duplicates=check_duplicates
+    )
     if ctx.verbose:
         click.echo("Packing from provided values: \n"
                    "labels: {labels} \n"
@@ -358,11 +383,102 @@ db/
                    "images: {images}".format(labels=labels, features=features, images=images))
     try:
         packed = packer.pack(labels, features, images)
+    except DuplicateFeatureVectorsException as exc:
+        handle_duplicates(ctx, exc.duplicates)
+        raise click.ClickException(str(exc))
     except ClassificationDatabaseException as exc:
         raise click.ClickException(str(exc))
     click.echo('Written to {}'.format(packed))
 
 
+@api.group()
+def rcdb_group():
+    pass
+
+
+@rcdb_group.command()
+@click.option('-r', '--retailer', help="Retailer codename", prompt=True)
+@click.option('-m', '--model', help="Model codename", prompt=True)
+@click.option('-c', '--concurrency', type=int, default=4)
+@click.argument('target', type=click.Path(exists=False, file_okay=False, dir_okay=True), default='.')
+@pass_rebotics_context
+def backup_and_download_rcdb(ctx, retailer, model, target, concurrency):
+    """
+    CLI command to trigger the upload
+    """
+    rcdb = ReboticsClassificationDatabase(ctx.provider)
+    # a flow that will request to do a backup of the retailer + model from the Admin
+    res = rcdb.create(retailer, model)
+    id_ = res['id']
+
+    # next flow wait for it to be processed calling the function to be done or not every N minutes
+    with Timer() as t:
+        while True:
+            res = rcdb.get(id_)
+            if res['status'] == RCDB.EXPORT_DONE:
+                file_url = res['backup']
+                break
+            if res['status'] == RCDB.EXPORT_ERROR:
+                raise click.ClickException("Creating and RCDB file failed")
+            sleep(5)
+
+    click.echo("Elapsed time: {}".format(t.elapsed_secs))
+
+    # flow that will download the rcdb file
+    file_io = remote_loaders.download(file_url, progress_bar=True)
+
+    # next flow that will unpack rcdb into target directory
+    target = pathlib.Path(target) / "rcdb_{}_{}_{}".format(retailer, model, id_)
+
+    packer = VirtualClassificationDatabasePacker(source=file_io)
+    download_vrcdb_file(packer, target, concurrency=concurrency)
+
+
+@rcdb_group.command()
+@click.option('-c', '--concurrency', type=int, default=4)
+@click.argument('target', type=click.Path(exists=False, file_okay=False, dir_okay=True), default='.')
+@click.argument('backup_ids', type=click.INT, nargs=-1)
+@pass_rebotics_context
+def download_rcdb(ctx, backup_ids, target, concurrency):
+    """
+    Download rcdb by id
+    :param ctx:
+    :param backup_ids:
+    :param target:
+    :param concurrency:
+    :return:
+    """
+    rcdb = ReboticsClassificationDatabase(ctx.provider)
+    target = pathlib.Path(target)
+
+    for backup_id in backup_ids:
+        res = rcdb.get(backup_id)
+
+        file_url = res['backup']
+        retailer = res['retailer']
+        model = res['facenet_model']
+
+        # flow that will download the rcdb file
+        file_io = remote_loaders.download(file_url, progress_bar=True)
+
+        # next flow that will unpack rcdb into target directory
+        download_folder = target / "rcdb_{}_{}_{}".format(retailer, model, backup_id)
+
+        packer = VirtualClassificationDatabasePacker(source=file_io)
+        download_vrcdb_file(packer, download_folder, concurrency=concurrency)
+
+
+@rcdb_group.command()
+@click.option('-c', '--concurrency', type=int, default=4)
+@click.argument('backup', type=click.Path(exists=True, file_okay=True, dir_okay=False))
+@click.argument('target', type=click.Path(exists=False, file_okay=False, dir_okay=True), default='.')
+@pass_rebotics_context
+def unpack_rcdb(backup, target, concurrency):
+    # TODO: check by filename and extension
+    packer = VirtualClassificationDatabasePacker(source=backup)
+    download_vrcdb_file(packer, target, concurrency=concurrency)
+
+
 api.add_command(shell, 'shell')
 api.add_command(roles, 'roles')
 api.add_command(configure, 'configure')
