@@ -880,6 +880,34 @@ def processing_spacial(ctx, target):
     click.echo("Saved into {}".format(target))
 
 
+@api.group(name='fv')
+def feature_vectors():
+    """
+    Feature vector flow
+    """
+    pass
+
+
+@feature_vectors.command('list')
+@pass_rebotics_context
+def feature_vectors_list(ctx):
+    fve = ctx.provider.feature_vectors_export()
+    data = fve.get()
+    if data:
+        ctx.format_result(data)
+
+
+@feature_vectors.command('export')
+@click.option('-b', '--batch-size', type=click.INT, default=50000)
+@click.option('-s', '--source-model', default='')
+@click.option('-r', '--result-model', default='previews-backup')
+@pass_rebotics_context
+def feature_vectors_export(ctx, source_model, result_model, batch_size):
+    fve = ctx.provider.feature_vectors_export()
+    result = fve.export(source_model=source_model, result_model=result_model, batch_size=batch_size)
+    ctx.format_result(result)
+
+
 api.add_command(shell, 'shell')
 api.add_command(roles, 'roles')
 api.add_command(configure, 'configure')
