@@ -284,7 +284,9 @@ class ReboticsBaseProvider(object):
 
     def is_retailer_identifier_used(self):
         header_keys = self.headers.keys()
-        return 'x-retailer-id' in header_keys and 'x-retailer-secret-key' in header_keys
+        return ('x-retailer-codename' in header_keys or 'x-retailer-id' in header_keys) \
+               and \
+               ('x-retailer-secret-key' in header_keys)
 
 
 class PageResult(object):
